#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <allegro.h>
#include "052a.h"
#include "052ac.h"
#include "052as.h"

int end_game;			/* flag we'll set to end the game */

BITMAP *dblbuffer;

#define MAX_CIRCLES     20
#define MAX_SQUARES     20

CIRCLE_T *moving_circle[MAX_CIRCLES];

SQUARE_T *moving_square[MAX_SQUARES];

int main(void)
{
    allegro_init();		/* initialise the Allegro library */
    init();			/* initialise the game */

    end_game = 0;		/* reset flag */

    do {			/* loop */
	input();		/* get input */
	process();		/* process it */
	output();		/* give output */
    } while (end_game == 0);	/* until the flag is set */

    shutdown();			/* shut down anything that needs it */
    allegro_exit();		/* just for luck */
    return 0;			/* tell the OS that all went well */
}
END_OF_MAIN()

void init(void)
{
    int f;

    srand(time(NULL));

    install_keyboard();

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) < 0) {
	printf("%s\n", allegro_error);
	exit(1);
    }
    dblbuffer = create_bitmap(SCREEN_W, SCREEN_H);
    if (dblbuffer == NULL) {
	allegro_exit();
	printf("Sorry, not enough memory");
	exit(2);
    }
    clear(dblbuffer);

    for (f = 0; f < MAX_CIRCLES; f++) {
	moving_circle[f] = circle_init(rand() % (SCREEN_W / 2),
				       rand() % (SCREEN_H / 2),
				       (rand() % 25) + 5,
				       (rand() % 15) + 1,
				       (rand() % 7) - 3,
				       (rand() % 7) - 3);
	if (moving_circle[f] == NULL) {
	    allegro_exit();
	    printf("Sorry, not enough memory");
	    exit(3);
	}
	circle_draw(moving_circle[f], dblbuffer);
    }

    for (f = 0; f < MAX_SQUARES; f++) {
	moving_square[f] = square_init(rand() % (SCREEN_W / 2),
				       rand() % (SCREEN_H / 2),
				       (rand() % 25) + 5,
				       (rand() % 25) + 5,
				       (rand() % 15) + 1,
				       (rand() % 7) - 3,
				       (rand() % 7) - 3);
	if (moving_square[f] == NULL) {
	    allegro_exit();
	    printf("Sorry, not enough memory");
	    exit(3);
	}
	square_draw(moving_square[f], dblbuffer);
    }

    show_double_buffer();
}

void input(void)
{
    if (keypressed())
	end_game++;
}

void process(void)
{
    int f;

    for (f = 0; f < MAX_CIRCLES; f++)
	circle_update(moving_circle[f]);

    for (f = 0; f < MAX_SQUARES; f++)
	square_update(moving_square[f]);
}

void output(void)
{
    int f;
    /* Notice the following, we have to separate circle_erase from the other
     * loop, or we could erase circles while others were drawing themselves.
     * The same with the squares.
     */
    for (f = 0; f < MAX_CIRCLES; f++)
	circle_erase(moving_circle[f], dblbuffer);

    for (f = 0; f < MAX_SQUARES; f++)
	square_erase(moving_square[f], dblbuffer);

    for (f = 0; f < MAX_CIRCLES; f++)
	circle_draw(moving_circle[f], dblbuffer);

    for (f = 0; f < MAX_SQUARES; f++)
	square_draw(moving_square[f], dblbuffer);

    show_double_buffer();
}

void shutdown(void)
{
    int f;

    for (f = 0; f < MAX_CIRCLES; f++)
	circle_destroy(moving_circle[f]);

    for (f = 0; f < MAX_SQUARES; f++)
	square_destroy(moving_square[f]);

    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    vsync();
    blit(dblbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
