#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <allegro.h>
#include "0531.h"
#include "0531c.h"

int end_game;			/* flag we'll set to end the game */
int how_many_circles;		// This contains the number of dinamic circles

BITMAP *dblbuffer;
CIRCLE_T *circles;

int main(void)
{
    allegro_init();		/* initialise the Allegro library */
    init();			/* initialise the game */

    end_game = 0;		/* reset flag */

    do {			/* loop */
	input();		/* get input */
	process();		/* process it */
	output();		/* give output */
    } while (end_game == 0);	/* until the flag is set */

    shutdown();			/* shut down anything that needs it */
    allegro_exit();		/* just for luck */
    return 0;			/* tell the OS that all went well */
}
END_OF_MAIN()

void reserve_memory(int num_circles)
{
    circles = (CIRCLE_T *) malloc(num_circles * sizeof(CIRCLE_T));

    if (circles == NULL) {
	printf("Sorry, not enough memory\n");
	exit(4);
    }
}

void free_memory(void)
{
    free(circles);
}

void init(void)
{
    int i;

    srand(time(NULL));

    printf("How many circles you want to see moving? : ");
    scanf("%d", &how_many_circles);

    if (how_many_circles < 1)
	how_many_circles = 1;

    reserve_memory(how_many_circles);

    install_keyboard();

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0) < 0) {
	printf("%s\n", allegro_error);
	exit(1);
    }
    dblbuffer = create_bitmap(SCREEN_W, SCREEN_H);
    if (dblbuffer == NULL) {
	allegro_exit();
	printf("Sorry, not enough memory");
	exit(2);
    }
    clear(dblbuffer);

    for (i = 0; i < how_many_circles; i++) {
	circle_init(rand() % SCREEN_W, rand() % SCREEN_H, (rand() % 25) + 5,
		    (rand() % 15) + 1,
		    (rand() % 7) - 3,
		    (rand() % 7) - 3,
		    &circles[i]);

	circle_draw(&circles[i], dblbuffer);
    }

    show_double_buffer();
}

void input(void)
{
    if (keypressed())
	end_game++;
}

void process(void)
{
    int f;

    for (f = 0; f < how_many_circles; f++)
	circle_update(&circles[f]);
}

void output(void)
{
    int f;
    /* Notice the following, we have to separate circle_erase from the other
     * loop, or we could erase circles while others were drawing themselves.
     */
    for (f = 0; f < how_many_circles; f++)
	circle_erase(&circles[f], dblbuffer);

    for (f = 0; f < how_many_circles; f++) {
	circle_draw(&circles[f], dblbuffer);
    }

    show_double_buffer();
}

void shutdown(void)
{
    free_memory();
    destroy_bitmap(dblbuffer);
}

void show_double_buffer(void)
{
    vsync();
    blit(dblbuffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
