#include <stdio.h>
#include <allegro.h>
#include "0825.h"

// Just make this global to let draw_options access it, bitmap too :-)...
int voice;
BITMAP *buffer;

int main(int argc, char *argv[])
{
    SAMPLE *the_sample;

    int max_voices, sound, looped, direction, volume, pan, freq;

    if (argc != 2) {
	printf("Usage: '0825 filename.wav (or .voc)'\n");
	return 1;
    }
    allegro_init();
    install_keyboard();
    install_timer();

    // Notice we need to detect this before install_sound()
    max_voices = detect_digi_driver(DIGI_AUTODETECT);

    if (max_voices > 0)
	printf("Your hardware supports %d simultaneous voices\n", max_voices);
    else {
	printf("Sorry, sound hardware not detected\n");
	return 2;
    }

    // Reserve voices. We only need one for digital, and none for midi.
    reserve_voices(1, -1);

    // Installs sound drivers
    sound = install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL);

    if (sound == 0)
	printf("Sound driver properly installed.\n");
    else {
	printf("Sound driver not installed:\n%s\n", allegro_error);
	return 3;
    }

    // Now load sound file...
    the_sample = load_sample(argv[1]);
    if (!the_sample) {
	printf("Error reading sound file '%s'\n", argv[1]);
	return 4;
    } else {
	printf("Sample loaded correctly...\n");
    }

    if (set_gfx_mode(GFX_AUTODETECT, 320, 200, 320, 200) != 0) {
	printf("Error setting video mode\n%s\n", allegro_error);
	return 5;
    }
    buffer = create_bitmap(SCREEN_W, SCREEN_H);
    clear(buffer);

    voice = allocate_voice(the_sample);

    /* This two store the playing mode. Add them to get the bitfield you need
     * when calling voice_set_playmode();
     */
    looped = PLAYMODE_PLAY;
    direction = PLAYMODE_FORWARD;
    volume = voice_get_volume(voice);
    pan = voice_get_pan(voice);
    freq = voice_get_frequency(voice);

    draw_options();

    while (!key[KEY_ESC] && !key[KEY_Q]) {
	draw_options();
	if (key[KEY_O]) {
	    looped = PLAYMODE_PLAY;
	    voice_stop(voice);
	    voice_set_playmode(voice, looped + direction);
	    voice_start(voice);
	}
	if (key[KEY_L]) {
	    looped = PLAYMODE_LOOP;
	    voice_stop(voice);
	    voice_set_playmode(voice, looped + direction);
	    voice_start(voice);
	}
	if (key[KEY_F]) {
	    direction = PLAYMODE_FORWARD;
	    voice_set_playmode(voice, looped + direction);
	}
	if (key[KEY_B]) {
	    direction = PLAYMODE_BACKWARD;
	    voice_set_playmode(voice, looped + direction);
	}
	if (key[KEY_I]) {
	    direction = PLAYMODE_BIDIR;
	    voice_set_playmode(voice, looped + direction);
	}
	if (key[KEY_PLUS_PAD] && volume < 255)
	    volume++;
	if (key[KEY_MINUS_PAD] && volume > 0)
	    volume--;

	if (key[KEY_RIGHT] && pan < 255)
	    pan++;
	if (key[KEY_LEFT] && pan > 0)
	    pan--;

	if (key[KEY_UP] && freq < 44000)
	    freq += 100;
	if (key[KEY_DOWN] && freq > 1000)
	    freq -= 100;

	voice_set_volume(voice, volume);
	voice_set_pan(voice, pan);
	voice_set_frequency(voice, freq);
    }

    return 0;
}
END_OF_MAIN()


void draw_options(void)
{
    clear(buffer);

    textout(buffer, font, "Sample characteristics:", 0, 0, WHITE);
    textprintf(buffer, font, 0, 10, WHITE,
	       "Volume: %d - Position: %d",
	       voice_get_volume(voice),
	       voice_get_position(voice));
    textprintf(buffer, font, 0, 20, WHITE,
	       "Pan: %d - Frequency: %d",
	       voice_get_pan(voice),
	       voice_get_frequency(voice));
    textout(buffer, font, "Play sample (O)nce, or (L)oop it.", 0, 40, WHITE);
    textout(buffer, font, "Playing mode (F)orward, (B)ackward", 0, 50, WHITE);
    textout(buffer, font, "or b(I)directional?", 0, 60, WHITE);
    textout(buffer, font, "+//- (keypad) set volume,", 0, 75, WHITE);
    textout(buffer, font, "left/right (cursors) pan sound,", 0, 85, WHITE);
    textout(buffer, font, "up/down (cursors) to change frequency.", 0, 95, WHITE);
    textout(buffer, font, "Press Q or ESC to quit", 0, 120, WHITE);

    blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}
